package com.kob.backend.consumer.utils;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.ArrayList;
import java.util.List;

/// 玩家信息
@Data
@AllArgsConstructor
@NoArgsConstructor
public class Player {
    private Integer id;
    private Integer sx;
    private Integer sy;
    // 玩家走过的路径中每一步的方向
    private List<Integer> steps;

    // 检验当前回合蛇的长度(蛇尾)是否需要增加
    public boolean checkTailIncreasing(int step) {
        // 前 10 回合每次都增加一节蛇尾,后面每 3 回合增加一节蛇尾
        if (step <= 10) return true;
        if (step % 3 == 1) return true;

        return false;
    }

    // 蛇身 Cell 列表:记录蛇身各部分坐标
    public List<Cell> getCells() {
        List<Cell> res = new ArrayList<>();

        // 四方向偏移量
        int[] dx = {-1, 0, 1, 0}, dy = {0, 1, 0, -1};
        int x = sx, y = sy; // 起始坐标
        int step = 0;   // 定义当前回合数: 开始时为 0 回合
        res.add(new Cell(x, y)); // 现将蛇头添加进 List
        for (int d : steps) {   // 使用下一步方向求出下一个结点的坐标
            x += dx[d];
            y += dy[d];
            res.add(new Cell(x, y)); // 添加新的身体结点
            step++;    // 回合数 +1
            // 判断蛇尾要不要增加:如果本回合蛇尾不增加,则将本回合新生成的蛇尾(列表的第0个元素)删掉
            if (!checkTailIncreasing(step)) {
                res.remove(0);
            }
        }
        return res;
    }
}
